/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.domain.comm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import de.bsvrz.iav.gllib.gllib.domain.GanglinieAnfrageTyp;
import de.bsvrz.iav.gllib.gllib.domain.GanglinienAnfrager;
import de.bsvrz.iav.gllib.gllib.domain.MessQuerschnitt;
import de.bsvrz.iav.gllib.gllib.domain.MqGanglinie;

/**
 * Basisklasse für Antworten zu Anfragen an den historischen Ganglinienspeicher.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class GlSpeicherAntwort {

    private final GanglinieAnfrageTyp typ;
    private final GanglinienAnfrager anfrager;
    private final String absenderZeichen;
    private final MessQuerschnitt messQuerschnitt;
    private final List<MqGanglinie> ganglinien = new ArrayList<>();

    private String fehlerMeldung;
    
    public GlSpeicherAntwort(GanglinieAnfrageTyp typ, GanglinienAnfrager anfrager, String absenderZeichen, MessQuerschnitt messQuerschnitt) {
        this.typ = typ;
        this.anfrager = anfrager;
        this.absenderZeichen = absenderZeichen;
        this.messQuerschnitt = messQuerschnitt;
    }

    public String getAbsenderZeichen() {
        return absenderZeichen;
    }

    public String getFehlerMeldung() {
        return fehlerMeldung;
    }

    public MessQuerschnitt getMessQuerschnitt() {
        return messQuerschnitt;
    }

    public void setFehlerMeldung(String fehlerMeldung) {
        this.fehlerMeldung = fehlerMeldung;
    }

    public void addGanglinie(MqGanglinie ganglinie) {
        ganglinien.add(ganglinie);
    }

    public void addGanglinien(Collection<MqGanglinie> neueGanglinien) {
        ganglinien.addAll(neueGanglinien);
    }
    
    public List<MqGanglinie> getGanglinien() {
        return Collections.unmodifiableList(ganglinien);
    }

    public GanglinienAnfrager getAnfrager() {
        return anfrager;
    }

    public GanglinieAnfrageTyp getTyp() {
        return typ;
    }
}
