/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.portals;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;

/**
 * Funktionen zum Prüfen der Konfigurationsvoraussetzungen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public final class DavChecker {

    private DavChecker() {
        // es gibt keine Instanzen der Klasse.
    }

    /**
     * Die Funktion prüft, ob über die mit der Datenverteilerverbindung verfügbare
     * Konfiguration die Mindestvoraussetzungen zur Verwendung der
     * Ganglinien-Funktionalität bietet.
     * 
     * Erwartet wird, dass der Konfigurationsbereich 'kb.tmGanglinienGlobal'
     * mindestens in Version 12 verfügbar ist.
     * 
     * Sind die Bedingungen nicht erfüllt wird eine {@link IllegalStateException}
     * geworfen.
     * 
     * @param dav die Datenverteilerverbindung
     */
    public static void checkConfiguration(ClientDavInterface dav) {
        ConfigurationArea kb = dav.getDataModel().getConfigurationArea("kb.tmGanglinienGlobal");
        if (kb == null) {
            throw new IllegalStateException(
                    "Der Konfigurationsbereich 'kb.tmGanglinienGlobal' ist in der Konfiguration nicht enthalten");
        }

        if (kb.getActiveVersion() < 12) {
            throw new IllegalStateException(
                    "Der Konfigurationsbereich 'kb.tmGanglinienGlobal' muss mindestens Version 12 haben");
        }
    }
}
