/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.portals;

import java.util.List;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.iav.gllib.gllib.domain.EreignisKalenderAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.EreignisTyp;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Anfrage an den Ereigniskalender zur Ermittlung der Zustandsübergänge in einem
 * festgelegten Zeitraum.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class DavEreignisKalenderAnfrageSender {

    private static final Debug LOGGER = Debug.getLogger();

    private ClientSenderInterface sender = new ClientSenderInterface() {

        @Override
        public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
            return true;
        }

        @Override
        public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {
            senderState = state;
            if (isBereit()) {
                sendPendingRequests();
            }
        }

    };

    private ClientDavInterface dav;
    private AttributeGroup anfrageAtg;
    private Aspect anfrageAsp;
    private byte senderState = ClientSenderInterface.STOP_SENDING;
    private Queue<ResultData> pendingRequests = new ConcurrentLinkedQueue<>();

    public DavEreignisKalenderAnfrageSender(ClientDavInterface dav) {
        this.dav = dav;
        anfrageAtg = dav.getDataModel().getAttributeGroup("atg.ereignisKalenderAnfrage");
        anfrageAsp = dav.getDataModel().getAspect("asp.anfrage");
        try {
            dav.subscribeSender(sender, dav.getLocalConfigurationAuthority(),
                    new DataDescription(anfrageAtg, anfrageAsp), SenderRole.sender());
        } catch (OneSubscriptionPerSendData e) {
            LOGGER.warning(e.getLocalizedMessage());
        }
    }

    private boolean isBereit() {
        return senderState == ClientSenderInterface.START_SENDING;
    }

    private void sendPendingRequests() {

        ResultData anfrage = null;
        do {
            anfrage = pendingRequests.poll();
            if (anfrage == null) {
                continue;
            }
            try {
                dav.sendData(anfrage);
            } catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
                LOGGER.warning(e.getLocalizedMessage());
            }
        } while (anfrage != null);
    }

    public void sendeAnfrage(EreignisKalenderAnfrage anfrage) {
        try {
            ResultData resultData = konvertiereAnfrage(anfrage);
            if (isBereit()) {
                dav.sendData(resultData);
            } else {
                pendingRequests.add(resultData);
            }
        } catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
            LOGGER.warning(e.getLocalizedMessage());
        }
    }

    private ResultData konvertiereAnfrage(EreignisKalenderAnfrage anfrage) {

        Data daten = dav.createData(anfrageAtg);

        daten.getReferenceValue("absenderId").setSystemObject(dav.getLocalApplicationObject());
        daten.getTextValue("absenderZeichen").setText(anfrage.getAbsenderZeichen());

        daten.getTimeValue("Anfangszeitpunkt").setMillis(anfrage.getIntervall().getStart());
        daten.getTimeValue("Endzeitpunkt").setMillis(anfrage.getIntervall().getEnd());

        List<String> raeumlicheGueltigkeit = anfrage.getRaeumlicheGueltigkeit();
        Array feld = daten.getArray("RäumlicheGültigkeit");
        feld.setLength(raeumlicheGueltigkeit.size());
        int i = 0;
        for (final String pid : raeumlicheGueltigkeit) {
            feld.getReferenceValue(i).setSystemObjectPid(pid);
        }

        daten.getUnscaledValue("EreignisTypenOption").set(anfrage.getEreignisTypenOption().getDavWert());

        List<EreignisTyp> ereignisTypReferenzen = anfrage.getEreignisTypen();
        feld = daten.getArray("EreignisTypReferenz");
        feld.setLength(ereignisTypReferenzen.size());
        i = 0;
        for (final EreignisTyp typ : ereignisTypReferenzen) {
            feld.getReferenceValue(i).setSystemObjectPid(typ.getPid());
        }

        return new ResultData(dav.getLocalConfigurationAuthority(), new DataDescription(anfrageAtg, anfrageAsp),
                dav.getTime(), daten);
    }

}
