/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.domain;

import java.util.Objects;

/**
 * 
 * Repräsentation eines Ereignistyps, dem Ganglinien zugeordnet werden können.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 *
 */
public final class EreignisTyp {

    public static final EreignisTyp KEIN_EREIGNIS_TYP = new EreignisTyp("");

    private final String pid;
    private long prioritaet;
    private EreignisTypLernParameter lernParameter;

    private EreignisTyp(String pid) {
        this.pid = pid;
    }

    public static EreignisTyp of(String pid) {
        return new EreignisTyp(pid);
    }

    public String getPid() {
        return pid;
    }

    @Override
    public String toString() {
        return pid;
    }

    @Override
    public int hashCode() {
        return Objects.hash(pid);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }

        EreignisTyp other = (EreignisTyp) obj;
        return Objects.equals(pid, other.pid);
    }

    public long getPrioritaet() {
        return prioritaet;
    }

    public void setPrioritaet(long prioritaet) {
        this.prioritaet = prioritaet;
    }

    public EreignisTypLernParameter getLernParameter() {
        if (lernParameter == null) {
            return EreignisTypLernParameter.NULL;
        }
        return lernParameter;
    }

    public void setLernParameter(EreignisTypLernParameter lernParameter) {
        this.lernParameter = lernParameter;
    }
}
