/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.portals;

import java.time.ZoneId;
import java.util.Collection;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.NumberArray;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.iav.gllib.gllib.domain.EreignisTyp;
import de.bsvrz.iav.gllib.gllib.domain.GanglinieAnfrageTyp;
import de.bsvrz.iav.gllib.gllib.domain.Messwerte;
import de.bsvrz.iav.gllib.gllib.domain.MqGanglinie;
import de.bsvrz.iav.gllib.gllib.domain.Stuetzstelle;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlAktualisierenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlAnlegenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlLesenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlLoeschenAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlSchreibenAnfrage;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * 
 * Implementierung eines Senders zum Versenden von Anfragen an den
 * Ganglinienspeicher über die Datenverteilerschnittstelle.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class DavGanglinieAnfrageSender {

	private static final Debug LOGGER = Debug.getLogger();

	private static ClientSenderInterface sender = new ClientSenderInterface() {

		@Override
		public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
			return false;
		}

		@Override
		public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {
			// Sendesteuerung wird nicht unterstützt
		}
	};

	private ClientDavInterface dav;

	private AttributeGroup anfrageAtg;

	private Aspect anfrageAsp;

	private SystemObject speicher;

	public DavGanglinieAnfrageSender(ClientDavInterface dav, SystemObject speicherObj) {
		this.dav = dav;
		this.speicher = speicherObj;
		anfrageAtg = dav.getDataModel().getAttributeGroup("atg.ganglinienSpeicherAnfrage");
		anfrageAsp = dav.getDataModel().getAspect("asp.anfrage");
		try {
			dav.subscribeSender(sender, speicher, new DataDescription(anfrageAtg, anfrageAsp), SenderRole.sender());
		} catch (OneSubscriptionPerSendData e) {
			LOGGER.warning(e.getLocalizedMessage());
		}

	}

	public void liesGanglinien(GlLesenAnfrage anfrage) {
		try {
			dav.sendData(konvertiereLesenAnfrage(anfrage));
		} catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
			LOGGER.warning(e.getLocalizedMessage());
		}
	}

	public void schreibeGanglinien(GlSchreibenAnfrage anfrage) {
		try {
			dav.sendData(konvertiereSchreibenAnfrage(anfrage));
		} catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
			LOGGER.warning(e.getLocalizedMessage());
		}
	}

	public void loescheGanglinien(GlLoeschenAnfrage anfrage) {
		try {
			dav.sendData(konvertiereLoeschenAnfrage(anfrage));
		} catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
			LOGGER.warning(e.getLocalizedMessage());
		}
	}

	public void erzeugeGanglinien(GlAnlegenAnfrage anfrage) {
		try {
			dav.sendData(konvertiereAnlegenAnfrage(anfrage));
		} catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
			LOGGER.warning(e.getLocalizedMessage());
		}
	}

	private ResultData konvertiereAnlegenAnfrage(GlAnlegenAnfrage anfrage) {
		Data daten = dav.createData(anfrageAtg);

		daten.getReferenceValue("absenderId").setSystemObject(dav.getLocalApplicationObject());
		daten.getTextValue("AbsenderZeichen").setText(anfrage.getAbsenderZeichen());

		daten.getItem("Anfrage").getReferenceValue("Messquerschnitt")
				.setSystemObjectPid(anfrage.getMessQuerschnitt().getPid());

		daten.getItem("Anfrage").getUnscaledValue("KommandoAnfrage").set(GanglinieAnfrageTyp.ANLEGEN.getCode());

		Array ganglinienArray = daten.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien");
		List<MqGanglinie> ganglinien = anfrage.getGanglinien();
		ganglinienArray.setLength(ganglinien.size());
		int idx = 0;
		for (MqGanglinie ganglinie : ganglinien) {
			fuelleGanglinie(ganglinienArray.getItem(idx++), ganglinie);
		}

		return new ResultData(speicher, new DataDescription(anfrageAtg, anfrageAsp), dav.getTime(), daten);
	}

	public void aktualisiereGanglinien(GlAktualisierenAnfrage anfrage) {
		try {
			dav.sendData(konvertiereAktualisierenAnfrage(anfrage));
		} catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
			LOGGER.warning(e.getLocalizedMessage());
		}
	}

	private ResultData konvertiereAktualisierenAnfrage(GlAktualisierenAnfrage anfrage) {
		Data daten = dav.createData(anfrageAtg);

		daten.getReferenceValue("absenderId").setSystemObject(dav.getLocalApplicationObject());
		daten.getTextValue("AbsenderZeichen").setText(anfrage.getAbsenderZeichen());

		daten.getItem("Anfrage").getReferenceValue("Messquerschnitt")
				.setSystemObjectPid(anfrage.getMessQuerschnitt().getPid());

		daten.getItem("Anfrage").getUnscaledValue("KommandoAnfrage").set(GanglinieAnfrageTyp.AKTUALISIEREN.getCode());

		Array ganglinienArray = daten.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien");
		List<MqGanglinie> ganglinien = anfrage.getGanglinien();
		ganglinienArray.setLength(ganglinien.size());
		int idx = 0;
		for (MqGanglinie ganglinie : ganglinien) {
			fuelleGanglinie(ganglinienArray.getItem(idx++), ganglinie);
		}

		return new ResultData(speicher, new DataDescription(anfrageAtg, anfrageAsp), dav.getTime(), daten);

	}

	private ResultData konvertiereLesenAnfrage(GlLesenAnfrage anfrage) {

		Data daten = dav.createData(anfrageAtg);

		daten.getReferenceValue("absenderId").setSystemObject(dav.getLocalApplicationObject());
		daten.getTextValue("AbsenderZeichen").setText(anfrage.getAbsenderZeichen());

		daten.getItem("Anfrage").getReferenceValue("Messquerschnitt")
				.setSystemObjectPid(anfrage.getMessQuerschnitt().getPid());

		Collection<EreignisTyp> anfrageEreignisTyp = anfrage.getEreignisTypen();
		Array ereignisTypArray = daten.getItem("Anfrage").getArray("EreignisTyp");
		ereignisTypArray.setLength(anfrageEreignisTyp.size());
		int idx = 0;
		for (EreignisTyp ereignisTyp : anfrageEreignisTyp) {
			ereignisTypArray.getReferenceValue(idx++).setSystemObjectPid(ereignisTyp.getPid());
		}
		daten.getItem("Anfrage").getUnscaledValue("KommandoAnfrage").set(GanglinieAnfrageTyp.LESEN.getCode());
		daten.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien").setLength(0);

		return new ResultData(speicher, new DataDescription(anfrageAtg, anfrageAsp), dav.getTime(), daten);

	}

	private ResultData konvertiereSchreibenAnfrage(GlSchreibenAnfrage anfrage) {

		Data daten = dav.createData(anfrageAtg);

		daten.getReferenceValue("absenderId").setSystemObject(dav.getLocalApplicationObject());
		daten.getTextValue("AbsenderZeichen").setText(anfrage.getAbsenderZeichen());

		daten.getItem("Anfrage").getReferenceValue("Messquerschnitt")
				.setSystemObjectPid(anfrage.getMessQuerschnitt().getPid());

		daten.getItem("Anfrage").getUnscaledValue("KommandoAnfrage").set(GanglinieAnfrageTyp.SCHREIBEN.getCode());

		Array ganglinienArray = daten.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien");
		List<MqGanglinie> ganglinien = anfrage.getGanglinien();
		ganglinienArray.setLength(ganglinien.size());
		int idx = 0;
		for (MqGanglinie ganglinie : ganglinien) {
			fuelleGanglinie(ganglinienArray.getItem(idx++), ganglinie);
		}

		return new ResultData(speicher, new DataDescription(anfrageAtg, anfrageAsp), dav.getTime(), daten);
	}

	private void fuelleGanglinie(Data item, MqGanglinie ganglinie) {
		item.getUnscaledValue("GanglinieId").set(ganglinie.getGanglinieId());
		item.getReferenceValue("EreignisTyp").setSystemObjectPid(ganglinie.getEreignisTyp().getPid());
		item.getUnscaledValue("Referenzganglinie").setText(ganglinie.isReferenz() ? "Ja" : "Nein");
		item.getUnscaledValue("AnzahlVerschmelzungen").set(ganglinie.getAnzahlVerschmelzungen());
		item.getTimeValue("LetzteVerschmelzung").setMillis(
				ganglinie.getLetzteVerschmelzung().atZone(ZoneId.systemDefault()).toInstant().toEpochMilli());
		item.getUnscaledValue("GanglinienTyp").set(ganglinie.getTyp().getCode());
		item.getUnscaledValue("GanglinienVerfahren").set(ganglinie.getApproximationDaK().getCode());
		item.getUnscaledValue("Ordnung").set(ganglinie.getBSplineOrdnung());

		Array stuetzStellenArray = item.getArray("Stützstelle");
		List<Stuetzstelle<Messwerte>> stuetzstellen = ganglinie.getStuetzstellen();
		stuetzStellenArray.setLength(stuetzstellen.size());
		int idx = 0;
		for (Stuetzstelle<Messwerte> stuetzstelle : stuetzstellen) {
			stuetzStellenArray.getItem(idx).getTimeValue("Zeit").setMillis(stuetzstelle.getZeitstempel());
			stuetzStellenArray.getItem(idx).getScaledValue("QLkw").set(stuetzstelle.getWert().getQLkwNichtNull());
			stuetzStellenArray.getItem(idx).getScaledValue("QKfz").set(stuetzstelle.getWert().getQKfzNichtNull());
			stuetzStellenArray.getItem(idx).getScaledValue("VPkw").set(stuetzstelle.getWert().getVPkwNichtNull());
			stuetzStellenArray.getItem(idx).getScaledValue("VLkw").set(stuetzstelle.getWert().getVLkwNichtNull());
			idx++;
		}
	}

	private ResultData konvertiereLoeschenAnfrage(GlLoeschenAnfrage anfrage) {
		Data daten = dav.createData(anfrageAtg);

		daten.getReferenceValue("absenderId").setSystemObject(dav.getLocalApplicationObject());
		daten.getTextValue("AbsenderZeichen").setText(anfrage.getAbsenderZeichen());

		daten.getItem("Anfrage").getReferenceValue("Messquerschnitt")
				.setSystemObjectPid(anfrage.getMessQuerschnitt().getPid());

		ReferenceArray ereignisTypArray = daten.getItem("Anfrage").getReferenceArray("EreignisTyp");
		ereignisTypArray.setLength(anfrage.getEreignisTypen().size());
		int idx = 0;
		for (EreignisTyp ereignisTyp : anfrage.getEreignisTypen()) {
			ereignisTypArray.getReferenceValue(idx++).setSystemObjectPid(ereignisTyp.getPid());
		}

		NumberArray ganglinienIdArray = daten.getItem("Anfrage").getUnscaledArray("GanglinienId");
		ganglinienIdArray.setLength(anfrage.getGanglinienId().size());
		idx = 0;
		for (long ganglinienId : anfrage.getGanglinienId()) {
			ganglinienIdArray.getValue(idx++).set(ganglinienId);
		}

		daten.getItem("Anfrage").getUnscaledValue("KommandoAnfrage").set(GanglinieAnfrageTyp.LOESCHEN.getCode());
		daten.getItem("Anfrage").getArray("ZuSpeicherndeGanglinien").setLength(0);

		return new ResultData(speicher, new DataDescription(anfrageAtg, anfrageAsp), dav.getTime(), daten);
	}

}
